@@ -526,8 +526,16 @@ SimRules::SimRules( int fd, QObject *p,  const QString& filename, HardwareManipu
              this, SLOT(dialCheck(QString,bool&)) );
 
     if ( machine ) {
-        connect( machine, SIGNAL(startIncomingCall(QString,QString)),
-                 _callManager, SLOT(startIncomingCall(QString,QString)) );
+        connect( machine, SIGNAL(startIncomingCall(QString,QString,QString)),
+                 _callManager, SLOT(startIncomingCall(QString,QString,QString)) );
+        connect ( _callManager, SIGNAL( callStatesChanged( QList<CallInfo> * ) ),
+                  machine, SLOT( callManagement( QList<CallInfo> * ) ) );
+        connect ( machine, SIGNAL( stateChangedToAlerting() ), _callManager,
+                SLOT( dialingToAlerting() ) );
+        connect ( machine, SIGNAL( stateChangedToConnected() ), _callManager,
+                SLOT( dialingToConnected() ) );
+        connect ( machine, SIGNAL( stateChangedToHangup( int ) ), _callManager,
+                SLOT( hangupRemote( int ) ) );
     }
 
     connect(this,SIGNAL(readyRead()),
@@ -546,8 +554,13 @@ SimRules::SimRules( int fd, QObject *p,  const QString& filename, HardwareManipu
     incomingUsed = 0;
     lineUsed = 0;
     defaultToolkitApp = toolkitApp = new DemoSimApplication( this, this );
+    conformanceApp = new ConformanceSimApplication( this, this );
     connect( _callManager, SIGNAL(controlEvent(QSimControlEvent)),
              toolkitApp, SLOT(controlEvent(QSimControlEvent)) );
+
+    simApps.append( toolkitApp );
+    simApps.append( conformanceApp );
+
     if ( machine )
         machine->handleNewApp();
 
@@ -839,9 +852,15 @@ void SimRules::tryReadCommand()
 
 void SimRules::destruct()
 {
-    if ( toolkitApp != defaultToolkitApp )
-        delete toolkitApp;
+    int count = simApps.count();
+
+    for ( int i = 0; i < count; i++ )
+        simApps.removeAt( 0 );
+
+    delete conformanceApp;
+    conformanceApp = NULL;
     delete defaultToolkitApp;
+    defaultToolkitApp = NULL;
     toolkitApp = NULL;
 
     if ( getMachine() )
@@ -877,13 +896,18 @@ HardwareManipulator * SimRules::getMachine() const
 
 void SimRules::setSimApplication( SimApplication *app )
 {
-    if ( toolkitApp != defaultToolkitApp )
-        delete toolkitApp;
-    toolkitApp = ( app ? app : defaultToolkitApp );
-    toolkitApp->start();
+    if ( toolkitApp == app )
+        return;
 
-    if ( getMachine() )
-        getMachine()->handleNewApp();
+    if ( toolkitApp )
+        toolkitApp->abort();
+
+    toolkitApp = app;
+}
+
+const QList<SimApplication *> SimRules::getSimApps()
+{
+    return simApps;
 }
 
 void SimRules::switchTo(const QString& name)
